
local StoneGateV5 = ...

-- Apply building asset processor
StoneGateV5:registerAssetProcessor("models/basic/basicLargeGate.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
-- Apply building asset processor
StoneGateV5:registerAssetProcessor("models/basic/basicBasementGate.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Large Gate prefab nodes
StoneGateV5:registerAssetId("models/basic/basicBasementGate.fbx/Prefab/basementGate/step_1_BGate_Tiling1Part", StoneGateSetPrefix .. "_BGATE_TILING1_PREFAB")

-- Register LargeGate prefab nodes
StoneGateV5:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_3_LGate_TopPart", StoneGateSetPrefix .. "_LGATE_TOP_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_3_LGateB_TopPart", StoneGateSetPrefix .. "_LGATEB_TOP_PREFAB")
--------------------------------------------------------------------------

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGateV5:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		Mover = { DataType = _MoverDefaultBuildingPart },
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1 }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	    }
	})
end

local defaultNodePrefixList = {
        StoneGateSetPrefix .. "_LGATE_PILLAR1",
        StoneGateSetPrefix .. "_LGATE_PILLAR2",
        StoneGateSetPrefix .. "_LGATE_PILLAR1P",
        StoneGateSetPrefix .. "_LGATE_PILLAR2P",
	StoneGateSetPrefix .. "_LGATE_TOP",
	StoneGateSetPrefix .. "_LGATEB_TOP",
	StoneGateSetPrefix .. "_LGATE_TILING1",
	StoneGateSetPrefix .. "_LGATE_TILING2",
	StoneGateSetPrefix .. "_LGATE_TILING1P",
	StoneGateSetPrefix .. "_LGATE_TILING2P",
        StoneGateSetPrefix .. "_LGATE_3_PILLAR1",
        StoneGateSetPrefix .. "_LGATE_3_PILLAR2",
        StoneGateSetPrefix .. "_LGATE_3_PILLAR3",
        StoneGateSetPrefix .. "_LGATE_3_PILLAR1P",
        StoneGateSetPrefix .. "_LGATE_3_PILLAR2P",
        StoneGateSetPrefix .. "_LGATE_3_PILLAR3P",
	StoneGateSetPrefix .. "_LGATE_TILING1_3",
	StoneGateSetPrefix .. "_LGATE_TILING1_3P",
        StoneGateSetPrefix .. "_LGATE_4_PILLAR1",
        StoneGateSetPrefix .. "_LGATE_4_PILLAR2",
        StoneGateSetPrefix .. "_LGATE_4_PILLAR3",
        StoneGateSetPrefix .. "_LGATE_4_PILLAR4",
        StoneGateSetPrefix .. "_LGATE_4_PILLAR1P",
        StoneGateSetPrefix .. "_LGATE_4_PILLAR2P",
        StoneGateSetPrefix .. "_LGATE_4_PILLAR3P",
        StoneGateSetPrefix .. "_LGATE_4_PILLAR4P",
	StoneGateSetPrefix .. "_LGATE_TILING1_4",
	StoneGateSetPrefix .. "_LGATE_TILING1_4P",
	StoneGateSetPrefix .. "_BGATE_TILING1"
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
end

function registerLargeGate2(_option)
    -- Register LargeGate prefab nodes
    StoneGateV5:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_Root" .. _option .. "Part", StoneGateSetPrefix .. "_LGATE_ROOT" .. _option .. "_PREFAB")
    StoneGateV5:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_Root" .. _option .. "Part/Pillar1" .. _option .. "_LGate_Part", StoneGateSetPrefix .. "_LGATE_PILLAR1" .. _option .. "_PREFAB")
    StoneGateV5:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_Root" .. _option .. "Part/Pillar2" .. _option .. "_LGate_Part", StoneGateSetPrefix .. "_LGATE_PILLAR2" .. _option .. "_PREFAB")
    StoneGateV5:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_1_LGate_Tiling1" .. _option .. "Part", StoneGateSetPrefix .. "_LGATE_TILING1" .. _option .. "_PREFAB")
    StoneGateV5:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_2_LGate_Tiling2" .. _option .. "Part", StoneGateSetPrefix .. "_LGATE_TILING2" .. _option .. "_PREFAB")

    -- Register the stone gate set main root part
    StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_LGATE_ROOT" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_LGATE_ROOT" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_LGATE_ROOT" .. _option .. "_DESC",
	Category = "CORE",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_LGATE_ROOT" .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_LGATE_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_LGATE_TILING2" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_LGATE_TILING1" .. _option .. "_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true,
		IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	}
    })

    -- Register grass clearing components
    StoneGateV5:registerPrefabComponent("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_Root" .. _option .. "Part", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 8, 8 }
    })

    -- Override building part components properties (generated by the building asset processor) to add building zones
    StoneGateV5:registerPrefabComponent("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_Root" .. _option .. "Part", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = false,
	BuildingZoneData = {
            DataType = "BUILDING_ZONE", 
            Polygon = polygon.createRectangle( { 0.1, 0.1 }, { 3.5, 3.5 } )
        }
    })

    StoneGateV5:registerPrefabComponent("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_Root" .. _option .. "Part/Pillar1" .. _option .. "_LGate_Part", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 2, 8 }
    })

    StoneGateV5:registerPrefabComponent("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_Root" .. _option .. "Part/Pillar2" .. _option .. "_LGate_Part", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 2, 8 }
    })
end

registerLargeGate2("") -- with stone ground
registerLargeGate2("P") -- with wooden ground

-- Register the stone gate set main root B part (deprecated, kept for compatibility)
StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_LGATEB_ROOT_PART",
	Name = StoneGateSetPrefix .. "_LGATEB_ROOT_NAME",
	Description = StoneGateSetPrefix .. "_LGATEB_ROOT_DESC",
	Category = "CORE",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_LGATE_ROOT_PREFAB",
		EndPart = StoneGateSetPrefix .. "_LGATEB_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_LGATE_TILING2_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_LGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true,
		IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	}
})

function registerLargeGate3(_option)
    -- Register LargeGate prefab nodes
    StoneGateV5:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_3_Root" .. _option .. "Part", StoneGateSetPrefix .. "_LGATE_3_ROOT" .. _option .. "_PREFAB")
    StoneGateV5:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_3_Root" .. _option .. "Part/Pillar1" .. _option .. "_LGate_3_Part", StoneGateSetPrefix .. "_LGATE_3_PILLAR1" .. _option .. "_PREFAB")
    StoneGateV5:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_3_Root" .. _option .. "Part/Pillar2" .. _option .. "_LGate_3_Part", StoneGateSetPrefix .. "_LGATE_3_PILLAR2" .. _option .. "_PREFAB")
    StoneGateV5:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_3_Root" .. _option .. "Part/Pillar3" .. _option .. "_LGate_3_Part", StoneGateSetPrefix .. "_LGATE_3_PILLAR3" .. _option .. "_PREFAB")
    StoneGateV5:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_1_LGate_Tiling1_3" .. _option .. "Part", StoneGateSetPrefix .. "_LGATE_TILING1_3" .. _option .. "_PREFAB")

    -- Register the stone gate set main root 3 accesses part
    StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_LGATE_3_ROOT" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_LGATE_3_ROOT" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_LGATE_3_ROOT" .. _option .. "_DESC",
	Category = "CORE",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_LGATE_3_ROOT" .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_LGATE_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_LGATE_TILING2" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_LGATE_TILING1_3" .. _option .. "_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true,
		IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	}
    })

    -- Register grass clearing components
    StoneGateV5:registerPrefabComponent("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_3_Root" .. _option .. "Part", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 8, 8 }
    })

    -- Override building part components properties (generated by the building asset processor) to add building zones
    StoneGateV5:registerPrefabComponent("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_3_Root" .. _option .. "Part", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = false,
	BuildingZoneData = {
            DataType = "BUILDING_ZONE", 
            Polygon = polygon.createRectangle( { 0.1, 0.1 }, { 3.5, 3.5 } )
        }
    })

    StoneGateV5:registerPrefabComponent("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_3_Root" .. _option .. "Part/Pillar1" .. _option .. "_LGate_3_Part", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 2, 8 }
    })

    StoneGateV5:registerPrefabComponent("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_3_Root" .. _option .. "Part/Pillar2" .. _option .. "_LGate_3_Part", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 2, 2 }
    })

    StoneGateV5:registerPrefabComponent("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_3_Root" .. _option .. "Part/Pillar3" .. _option .. "_LGate_3_Part", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 2, 2 }
    })
end

registerLargeGate3("") -- with stone ground
registerLargeGate3("P") -- with wooden ground

function registerLargeGate4(_option)
    -- Register LargeGate prefab nodes
    StoneGateV5:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_4_Root" .. _option .. "Part", StoneGateSetPrefix .. "_LGATE_4_ROOT" .. _option .. "_PREFAB")
    StoneGateV5:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_4_Root" .. _option .. "Part/Pillar1" .. _option .. "_LGate_4_Part", StoneGateSetPrefix .. "_LGATE_4_PILLAR1" .. _option .. "_PREFAB")
    StoneGateV5:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_4_Root" .. _option .. "Part/Pillar2" .. _option .. "_LGate_4_Part", StoneGateSetPrefix .. "_LGATE_4_PILLAR2" .. _option .. "_PREFAB")
    StoneGateV5:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_4_Root" .. _option .. "Part/Pillar3" .. _option .. "_LGate_4_Part", StoneGateSetPrefix .. "_LGATE_4_PILLAR3" .. _option .. "_PREFAB")
    StoneGateV5:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_4_Root" .. _option .. "Part/Pillar4" .. _option .. "_LGate_4_Part", StoneGateSetPrefix .. "_LGATE_4_PILLAR4" .. _option .. "_PREFAB")
    StoneGateV5:registerAssetId("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_1_LGate_Tiling1_4" .. _option .. "Part", StoneGateSetPrefix .. "_LGATE_TILING1_4" .. _option .. "_PREFAB")

    -- Register the stone gate set main root 4 accesses part
    StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_LGATE_4_ROOT" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_LGATE_4_ROOT" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_LGATE_4_ROOT" .. _option .. "_DESC",
	Category = "CORE",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_LGATE_4_ROOT" .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_LGATE_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_LGATE_TILING2" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_LGATE_TILING1_4" .. _option .. "_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true,
		IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	}
    })

    -- Register grass clearing components
    StoneGateV5:registerPrefabComponent("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_4_Root" .. _option .. "Part", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 8, 8 }
    })

    -- Override building part components properties (generated by the building asset processor) to add building zones
    StoneGateV5:registerPrefabComponent("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_4_Root" .. _option .. "Part", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = false,
	BuildingZoneData = {
            DataType = "BUILDING_ZONE", 
            Polygon = polygon.createRectangle( { 0.1, 0.1 }, { 3.5, 3.5 } )
        }
    })

    StoneGateV5:registerPrefabComponent("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_4_Root" .. _option .. "Part/Pillar1" .. _option .. "_LGate_4_Part", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 2, 2 }
    })

    StoneGateV5:registerPrefabComponent("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_4_Root" .. _option .. "Part/Pillar2" .. _option .. "_LGate_4_Part", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 2, 2 }
    })

    StoneGateV5:registerPrefabComponent("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_4_Root" .. _option .. "Part/Pillar3" .. _option .. "_LGate_4_Part", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 2, 2 }
    })

    StoneGateV5:registerPrefabComponent("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_0_LGate_4_Root" .. _option .. "Part/Pillar4" .. _option .. "_LGate_4_Part", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 2, 2 }
    })
end

registerLargeGate4("") -- with stone ground
registerLargeGate4("P") -- with wooden ground

--------------------------------------------------------------------------
-- Register flag assigner component on the collider's type to 'PLATFORM'
--StoneGateV5:configurePrefabFlagList("models/basic/basicLargeGate.fbx/Prefab/largeGate/step_3_LGate_TopPart", { "PLATFORM" })
