
local StoneGateV5 = ...

-- Apply building asset processor
StoneGateV5:registerAssetProcessor("models/basic/basicOctogonalTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Octogonal Tower materials
StoneGateV5:registerAssetId("models/basic/basicOctogonalTower.fbx/Materials/redRoof", StoneGateSetPrefix .. "_OTOWER_RED_ROOF_MATERIAL")
StoneGateV5:registerAssetId("models/basic/basicOctogonalTower.fbx/Materials/blueRoof", StoneGateSetPrefix .. "_OTOWER_BLUE_ROOF_MATERIAL")
StoneGateV5:registerAssetId("models/basic/basicOctogonalTower.fbx/Materials/brownRoof", StoneGateSetPrefix .. "_OTOWER_BROWN_ROOF_MATERIAL")
StoneGateV5:registerAssetId("models/basic/basicOctogonalTower.fbx/Materials/greyRoof", StoneGateSetPrefix .. "_OTOWER_GREY_ROOF_MATERIAL")
StoneGateV5:registerAssetId("models/basic/basicOctogonalTower.fbx/Materials/grey2Roof", StoneGateSetPrefix .. "_OTOWER_GREY2_ROOF_MATERIAL")

-- Register Octogonal Tower prefab nodes
StoneGateV5:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_3_Octogonal_TopPart", StoneGateSetPrefix .. "_OTOWER_TOP_PREFAB")
--------------------------------------------------------------------------

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGateV5:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		Mover = { DataType = "BUILDING_PART_MOVER" },
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1 }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	    }
	})
end

local defaultNodePrefixList = {
	StoneGateSetPrefix .. "_OTOWER_TOP",
	StoneGateSetPrefix .. "_OTOWER_TILING1",
	StoneGateSetPrefix .. "_OTOWER_TILING2",
	StoneGateSetPrefix .. "_OTOWER_TILING3",
	StoneGateSetPrefix .. "_OTOWER_TILING1B",
	StoneGateSetPrefix .. "_OTOWER_TILING2B",
	StoneGateSetPrefix .. "_OTOWER_TILING3B"
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
end

function registerOctogonalTower(_option)
    -- Register Octogonal Tower prefab nodes
    StoneGateV5:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_0_Octogonal_Root" .. _option .. "Part", StoneGateSetPrefix .. "_OTOWER_ROOT" .. _option .. "_PREFAB")
    StoneGateV5:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_1_Octogonal_Tiling1" .. _option .. "Part", StoneGateSetPrefix .. "_OTOWER_TILING1" .. _option .. "_PREFAB")
    StoneGateV5:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_2_Octogonal_Tiling2" .. _option .. "Part", StoneGateSetPrefix .. "_OTOWER_TILING2" .. _option .. "_PREFAB")
    StoneGateV5:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_2_Octogonal_Tiling3" .. _option .. "Part", StoneGateSetPrefix .. "_OTOWER_TILING3" .. _option .. "_PREFAB")

    -- Register the stone gate set main root part
    StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_OTOWER_ROOT" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_OTOWER_ROOT" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_OTOWER_ROOT" .. _option .. "_DESC",
	Category = "CORE",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_OTOWER_ROOT" .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_OTOWER_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_OTOWER_TILING2" .. _option .. "_PART",
			StoneGateSetPrefix .. "_OTOWER_TILING3" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_OTOWER_TILING1" .. _option .. "_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true,
		IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	},
        VillagerCapacity = 12,
	AssetBuildingFunction = "BUILDING_FUNCTION_ASSIGNABLE_LORD_MANOR"
    })

    -- Register grass clearing components
    StoneGateV5:registerPrefabComponent("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_0_Octogonal_Root" .. _option .. "Part", {
	DataType = "COMP_DIRT_CIRCLE",
	HardRadius = 4.6,
        SoftRadius = 4.6,
        IsPermanent = true
    })

    -- Override building part components properties (generated by the building asset processor) to add building zones
    StoneGateV5:registerPrefabComponent("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_0_Octogonal_Root" .. _option .. "Part", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 9, 9 }
    })
end

registerOctogonalTower("") -- with stone ground
registerOctogonalTower("B") -- with wooden ground

--------------------------------------------------------------------------
-- Register flag assigner component on the collider's type to 'PLATFORM'

--StoneGateV5:configurePrefabFlagList("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_3_Octogonal_TopPart", { "PLATFORM" })
 
function registerRoofPart(_nameFbx, _nameAsset)
    StoneGateV5:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration roof part
    StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 5 }
		}
	}
    })

    StoneGateV5:registerPrefabComponent("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the roofs)
	BuildingZone = { 0, 0 }
    })
end

registerRoofPart( "OctogonalRoof", "OTOWER_ROOF" )
registerRoofPart( "OctogonalRoofB", "OTOWER_ROOFB" )

---- Ramparts...

-- Apply building asset processor
StoneGateV5:registerAssetProcessor("models/ramparts/rampartOctogonalTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

function registerRampartPart(_nameFbx, _nameAsset)
    StoneGateV5:registerAssetId("models/ramparts/rampartOctogonalTower.fbx/Prefab/octogonalTower/step_4_Octogonal_" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_OTOWER_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration rampart part
    StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_OTOWER_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_OTOWER_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_OTOWER_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_OTOWER_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	}
    })

    StoneGateV5:registerPrefabComponent("models/ramparts/rampartOctogonalTower.fbx/Prefab/octogonalTower/step_4_Octogonal_" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
--	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the roofs)
	BuildingZone = { 0, 0 }
    })
end

registerRampartPart( "Rampart", "RAMPART" )
registerRampartPart( "RampartB", "RAMPART_B" )
registerRampartPart( "RampartC", "RAMPART_C" )
